{% extends "wiki/_base.html" %}

{% from "macros/box.html" import m_box_content %}

{% block content %}
  {% call m_box_content(_("All pages")) %}
    <ul style="list-style: none;">
      {% for page in pages %}
        <li><a href="{{ url_for(".page", community_id=g.community.slug, title=page.title) }}">{{ page.title }}</a></li>
      {% endfor %}
    </ul>
  {% endcall %}
{% endblock %}
