// select {
//   -webkit-appearance: none;
//   -webkit-padding-end: 20px;
//   -webkit-padding-start: 6px;

//   -moz-appearance: none;
//   text-indent: 0.01px; /* Removes default arrow from firefox*/
//   text-overflow: "";  /*Removes default arrow from firefox*/

//   /* OVERRIDE */
//   background-image: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABMAAAAICAYAAAAbQcSUAAAAWklEQVQokWNgoAOIAuI0PDiKaJMSgYCZmfkbkPkfHYPEQfJEG/b//3+FBQsWLGRjY/uJbBCIDxIHyRNtGDYDyTYI3UA+Pr4vFBmEbODbt2+bKDYIyUBWYtQBAIRzRP/XKJ//AAAAAElFTkSuQmCC) !important;
//   background-position: right center;
//   background-repeat: no-repeat;
// }

label::first-letter {
  text-transform: uppercase;
}

/*
form [role="tabpanel"] > .form-group:nth-of-type(even) {
  margin-top: @form-group-margin-bottom / 2;
  margin-bottom: @form-group-margin-bottom / 2;
}

form [role="tabpanel"] > .form-group:nth-of-type(odd) {
  background-color: @table-bg-accent;
  padding-top: @form-group-margin-bottom / 2;
  padding-bottom: @form-group-margin-bottom / 2;

}
*/

.form-group .form-group {
  /* fieldform */
  margin-bottom: 5px;

  > label {
    /* reset margin bottom when primary form is form-horizontal */
    margin-bottom: 5px;
  }

  .help-block {
    margin-bottom: 0;
  }
}

form [role="tablist"] .required::after,
.form-group.required > label::after,
label.required::after {
  content: "\2605"; // U+2605: BLACK STAR ★
  vertical-align: super;
  font-size: 85%;
  margin-left: 0.25ex;
  color: #b00;
}

/* to disable resize on textareas */
.not-resizeable {
  resize: none;
}

.resizeable-vertical {
  resize: vertical;
}

.form-control.editor {
  height: auto;
}

/* tabular widget */
table.tabular-form {
  td,
  th {
    padding: 0.1em 0.15em;
  }
}

/* display btn as inline text */
.btn-inline {
  display: inline;
  margin: 0;
  border: 0;
  padding: 0;
  background: inherit;
}

/* fileinput */
.btn-file {
  position: relative;
  overflow: hidden;

  input[type="file"] {
    position: absolute;
    top: 0;
    right: 0;
    min-width: 100%;
    min-height: 100%;
    font-size: 999px;
    text-align: right;
    filter: alpha(opacity=0);
    opacity: 0;
    background: red;
    cursor: inherit;
    display: block;
  }
}

.file-input {
  .clearfix();

  .selected-files {
    margin-top: 0.5em;
    margin-bottom: 0.5em;
  }

  .file-item {
    .make-xs-column(12);
    .make-md-column(6);
    .well();
    .well-sm();

    margin-bottom: 0;

    canvas,
    img {
      vertical-align: middle;
    }
  }

  .file-item-existing button {
    float: right;
    font-size: 200%;
  }
}

.form-inline {
  & .file-input,
  & .selected-files {
    display: inline-block;
    vertical-align: middle;
    width: auto;
  }

  & .selected-files {
    margin: auto 0;

    .file-item {
      width: auto;
    }
  }
}

/* fix alternative select2-bootstrap.css from
https://github.com/fk/select2-bootstrap-css/ */
.form-inline .select2-container.form-control {
  display: inline-block;
}

.dynamic-row-widget.model-field-list tbody > tr > td {
  border-top-width: 4px;
}
